@@ -22,27 +22,28 @@ import (
 	"io/ioutil"
 	"os"
 	"testing"
+	"time"
 
+	"github.com/pkg/errors"
+	monitoringv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
 	cephv1 "github.com/rook/rook/pkg/apis/ceph.rook.io/v1"
-	"github.com/rook/rook/pkg/apis/rook.io"
 	"github.com/rook/rook/pkg/client/clientset/versioned/scheme"
 	"github.com/rook/rook/pkg/clusterd"
 	cephclient "github.com/rook/rook/pkg/daemon/ceph/client"
+	"github.com/rook/rook/pkg/operator/ceph/controller"
 	cephver "github.com/rook/rook/pkg/operator/ceph/version"
-	"sigs.k8s.io/controller-runtime/pkg/client/fake"
-
-	monitoringv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
 	testopk8s "github.com/rook/rook/pkg/operator/k8sutil/test"
 	testop "github.com/rook/rook/pkg/operator/test"
 	exectest "github.com/rook/rook/pkg/util/exec/test"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-	"github.com/tevino/abool"
 	apps "k8s.io/api/apps/v1"
+	corev1 "k8s.io/api/core/v1"
 	policyv1 "k8s.io/api/policy/v1"
 	policyv1beta1 "k8s.io/api/policy/v1beta1"
-	"k8s.io/apimachinery/pkg/api/errors"
+	kerrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 )
 
 func TestStartMgr(t *testing.T) {
@@ -58,7 +59,7 @@ func TestStartMgr(t *testing.T) {
 			return "{\"key\":\"mysecurekey\"}", nil
 		},
 	}
-	waitForDeploymentToStart = func(clusterdContext *clusterd.Context, deployment *apps.Deployment) error {
+	waitForDeploymentToStart = func(ctx context.Context, clusterdContext *clusterd.Context, deployment *apps.Deployment) error {
 		logger.Infof("simulated mgr deployment starting")
 		return nil
 	}
@@ -74,20 +75,20 @@ func TestStartMgr(t *testing.T) {
 
 	defer os.RemoveAll(configDir)
 	ctx := &clusterd.Context{
-		Executor:                   executor,
-		ConfigDir:                  configDir,
-		Clientset:                  clientset,
-		RequestCancelOrchestration: abool.New(),
-		Client:                     cl}
+		Executor:  executor,
+		ConfigDir: configDir,
+		Clientset: clientset,
+		Client:    cl,
+	}
 	ownerInfo := cephclient.NewMinimumOwnerInfo(t)
-	clusterInfo := &cephclient.ClusterInfo{Namespace: "ns", FSID: "myfsid", OwnerInfo: ownerInfo, CephVersion: cephver.CephVersion{Major: 16, Minor: 2, Build: 5}}
+	clusterInfo := &cephclient.ClusterInfo{Namespace: "ns", FSID: "myfsid", OwnerInfo: ownerInfo, CephVersion: cephver.CephVersion{Major: 16, Minor: 2, Build: 5}, Context: context.TODO()}
 	clusterInfo.SetName("test")
 	clusterSpec := cephv1.ClusterSpec{
-		Annotations:        map[rook.KeyType]rook.Annotations{cephv1.KeyMgr: {"my": "annotation"}},
-		Labels:             map[rook.KeyType]rook.Labels{cephv1.KeyMgr: {"my-label-key": "value"}},
+		Annotations:        map[cephv1.KeyType]cephv1.Annotations{cephv1.KeyMgr: {"my": "annotation"}},
+		Labels:             map[cephv1.KeyType]cephv1.Labels{cephv1.KeyMgr: {"my-label-key": "value"}},
 		Dashboard:          cephv1.DashboardSpec{Enabled: true, SSL: true},
 		Mgr:                cephv1.MgrSpec{Count: 1},
-		PriorityClassNames: map[rook.KeyType]string{cephv1.KeyMgr: "my-priority-class"},
+		PriorityClassNames: map[cephv1.KeyType]string{cephv1.KeyMgr: "my-priority-class"},
 		DataDirHostPath:    "/var/lib/rook/",
 	}
 	c := New(ctx, clusterInfo, clusterSpec, "myversion")
@@ -95,7 +96,7 @@ func TestStartMgr(t *testing.T) {
 
 	// start a basic service
 	err = c.Start()
-	assert.Nil(t, err)
+	assert.NoError(t, err)
 	validateStart(t, c)
 	assert.ElementsMatch(t, []string{}, testopk8s.DeploymentNamesUpdated(deploymentsUpdated))
 	testopk8s.ClearDeploymentsUpdated(deploymentsUpdated)
@@ -103,7 +104,7 @@ func TestStartMgr(t *testing.T) {
 	c.spec.Dashboard.URLPrefix = "/test"
 	c.spec.Dashboard.Port = 12345
 	err = c.Start()
-	assert.Nil(t, err)
+	assert.NoError(t, err)
 	validateStart(t, c)
 	assert.ElementsMatch(t, []string{"rook-ceph-mgr-a"}, testopk8s.DeploymentNamesUpdated(deploymentsUpdated))
 	testopk8s.ClearDeploymentsUpdated(deploymentsUpdated)
@@ -113,19 +114,19 @@ func TestStartMgr(t *testing.T) {
 	c.spec.Dashboard.Enabled = false
 	// delete the previous mgr since the mocked test won't update the existing one
 	err = c.context.Clientset.AppsV1().Deployments(c.clusterInfo.Namespace).Delete(context.TODO(), "rook-ceph-mgr-a", metav1.DeleteOptions{})
-	assert.Nil(t, err)
+	assert.NoError(t, err)
 	err = c.Start()
-	assert.Nil(t, err)
+	assert.NoError(t, err)
 	validateStart(t, c)
 
 	c.spec.Mgr.Count = 1
 	c.spec.Dashboard.Enabled = false
 	// clean the previous deployments
 	err = c.context.Clientset.AppsV1().Deployments(c.clusterInfo.Namespace).Delete(context.TODO(), "rook-ceph-mgr-a", metav1.DeleteOptions{})
-	assert.Nil(t, err)
-	assert.Nil(t, err)
+	assert.NoError(t, err)
+	assert.NoError(t, err)
 	err = c.Start()
-	assert.Nil(t, err)
+	assert.NoError(t, err)
 	validateStart(t, c)
 }
 
@@ -135,7 +136,7 @@ func validateStart(t *testing.T, c *Cluster) {
 		logger.Infof("Looking for cephmgr replica %d", i)
 		daemonName := mgrNames[i]
 		d, err := c.context.Clientset.AppsV1().Deployments(c.clusterInfo.Namespace).Get(context.TODO(), fmt.Sprintf("rook-ceph-mgr-%s", daemonName), metav1.GetOptions{})
-		assert.Nil(t, err)
+		assert.NoError(t, err)
 		assert.Equal(t, map[string]string{"my": "annotation"}, d.Spec.Template.Annotations)
 		assert.Contains(t, d.Spec.Template.Labels, "my-label-key")
 		assert.Equal(t, "my-priority-class", d.Spec.Template.Spec.PriorityClassName)
@@ -160,7 +161,7 @@ func validateStart(t *testing.T, c *Cluster) {
 
 func validateServices(t *testing.T, c *Cluster) {
 	_, err := c.context.Clientset.CoreV1().Services(c.clusterInfo.Namespace).Get(context.TODO(), "rook-ceph-mgr", metav1.GetOptions{})
-	assert.Nil(t, err)
+	assert.NoError(t, err)
 
 	ds, err := c.context.Clientset.CoreV1().Services(c.clusterInfo.Namespace).Get(context.TODO(), "rook-ceph-mgr-dashboard", metav1.GetOptions{})
 	if c.spec.Dashboard.Enabled {
@@ -173,10 +174,66 @@ func validateServices(t *testing.T, c *Cluster) {
 			assert.Equal(t, ds.Spec.Ports[0].Port, int32(c.spec.Dashboard.Port))
 		}
 	} else {
-		assert.True(t, errors.IsNotFound(err))
+		assert.True(t, kerrors.IsNotFound(err))
 	}
 }
 
+func TestUpdateServiceSelectors(t *testing.T) {
+	clientset := testop.New(t, 3)
+	ctx := &clusterd.Context{Clientset: clientset}
+	clusterInfo := cephclient.AdminTestClusterInfo("mycluster")
+	spec := cephv1.ClusterSpec{
+		Dashboard: cephv1.DashboardSpec{
+			Enabled: true,
+			Port:    7000,
+		},
+	}
+	c := &Cluster{spec: spec, context: ctx, clusterInfo: clusterInfo}
+
+	t.Run("initial active daemon", func(t *testing.T) {
+		activeDaemon := "a"
+		err := c.reconcileServices(activeDaemon)
+		assert.NoError(t, err)
+		validateServiceActiveDaemon(t, c, activeDaemon, 2, 0)
+	})
+
+	t.Run("update active daemon", func(t *testing.T) {
+		activeDaemon := "b"
+		err := c.updateServiceSelectors(activeDaemon)
+		assert.NoError(t, err)
+		validateServiceActiveDaemon(t, c, activeDaemon, 2, 0)
+	})
+
+	t.Run("skip non-mgr services", func(t *testing.T) {
+		svc := corev1.Service{ObjectMeta: metav1.ObjectMeta{Name: "mysvc"}}
+		_, err := c.context.Clientset.CoreV1().Services(c.clusterInfo.Namespace).Create(clusterInfo.Context, &svc, metav1.CreateOptions{})
+		assert.NoError(t, err)
+
+		activeDaemon := "c"
+		err = c.updateServiceSelectors(activeDaemon)
+		assert.NoError(t, err)
+		validateServiceActiveDaemon(t, c, activeDaemon, 2, 1)
+	})
+}
+
+func validateServiceActiveDaemon(t *testing.T, c *Cluster, activeDaemon string, expectedUpdated, expectedSkipped int) {
+	services, err := c.context.Clientset.CoreV1().Services(c.clusterInfo.Namespace).List(c.clusterInfo.Context, metav1.ListOptions{})
+	assert.NoError(t, err)
+	skipped := 0
+	updated := 0
+	for _, service := range services.Items {
+		if service.Labels["app"] == "rook-ceph-mgr" {
+			updated++
+			assert.Equal(t, activeDaemon, service.Spec.Selector[controller.DaemonIDLabel])
+		} else {
+			skipped++
+			assert.Equal(t, "", service.Spec.Selector[controller.DaemonIDLabel])
+		}
+	}
+	assert.Equal(t, expectedUpdated, updated)
+	assert.Equal(t, expectedSkipped, skipped)
+}
+
 func TestMgrSidecarReconcile(t *testing.T) {
 	activeMgr := "a"
 	calledMgrStat := false
@@ -207,8 +264,7 @@ func TestMgrSidecarReconcile(t *testing.T) {
 		ConfigDir: configDir,
 		Clientset: clientset,
 	}
-	ownerInfo := cephclient.NewMinimumOwnerInfo(t)
-	clusterInfo := &cephclient.ClusterInfo{Namespace: "ns", OwnerInfo: ownerInfo}
+	clusterInfo := cephclient.AdminTestClusterInfo("mycluster")
 	clusterInfo.SetName("test")
 	c := &Cluster{spec: spec, context: ctx, clusterInfo: clusterInfo}
 
@@ -229,7 +285,7 @@ func TestMgrSidecarReconcile(t *testing.T) {
 	assert.True(t, calledMgrStat)
 	assert.False(t, calledMgrDump)
 	_, err = c.context.Clientset.CoreV1().Services(c.clusterInfo.Namespace).Get(context.TODO(), "rook-ceph-mgr", metav1.GetOptions{})
-	assert.True(t, errors.IsNotFound(err))
+	assert.True(t, kerrors.IsNotFound(err))
 
 	// nothing is updated when the requested mgr is not the active mgr
 	activeMgr = "b"
@@ -270,17 +326,20 @@ func TestConfigureModules(t *testing.T) {
 					}
 					lastModuleConfigured = args[3]
 				}
-				if args[0] == "config" && args[1] == "set" && args[2] == "global" {
-					configSettings[args[3]] = args[4]
-				}
 			}
 			return "", nil //return "{\"key\":\"mysecurekey\"}", nil
 		},
+		MockExecuteCommandWithTimeout: func(timeout time.Duration, command string, args ...string) (string, error) {
+			if args[0] == "config" && args[1] == "set" && args[2] == "global" {
+				configSettings[args[3]] = args[4]
+			}
+			return "", nil
+		},
 	}
 
 	clientset := testop.New(t, 3)
 	context := &clusterd.Context{Executor: executor, Clientset: clientset}
-	clusterInfo := &cephclient.ClusterInfo{Namespace: "ns"}
+	clusterInfo := cephclient.AdminTestClusterInfo("mycluster")
 	c := &Cluster{
 		context:     context,
 		clusterInfo: clusterInfo,
@@ -377,3 +436,58 @@ func TestApplyMonitoringLabels(t *testing.T) {
 	applyMonitoringLabels(c, sm)
 	assert.Nil(t, sm.Spec.Endpoints[0].RelabelConfigs)
 }
+
+func TestCluster_enableBalancerModule(t *testing.T) {
+	c := &Cluster{
+		context:     &clusterd.Context{Executor: &exectest.MockExecutor{}, Clientset: testop.New(t, 3)},
+		clusterInfo: cephclient.AdminTestClusterInfo("mycluster"),
+	}
+
+	t.Run("on octopus we configure the balancer AND enable the upmap mode", func(t *testing.T) {
+		c.clusterInfo.CephVersion = cephver.Octopus
+		executor := &exectest.MockExecutor{
+			MockExecuteCommandWithOutput: func(command string, args ...string) (string, error) {
+				logger.Infof("Command: %s %v", command, args)
+				if command == "ceph" {
+					if args[0] == "osd" && args[1] == "set-require-min-compat-client" {
+						return "", nil
+					}
+					if args[0] == "balancer" && args[1] == "mode" {
+						return "", nil
+					}
+					if args[0] == "balancer" && args[1] == "on" {
+						return "", nil
+					}
+				}
+				return "", errors.New("unknown command")
+			},
+		}
+		c.context.Executor = executor
+		err := c.enableBalancerModule()
+		assert.NoError(t, err)
+	})
+
+	t.Run("on pacific we configure the balancer ONLY and don't set a mode", func(t *testing.T) {
+		c.clusterInfo.CephVersion = cephver.Pacific
+		executor := &exectest.MockExecutor{
+			MockExecuteCommandWithOutput: func(command string, args ...string) (string, error) {
+				logger.Infof("Command: %s %v", command, args)
+				if command == "ceph" {
+					if args[0] == "osd" && args[1] == "set-require-min-compat-client" {
+						return "", nil
+					}
+					if args[0] == "balancer" && args[1] == "mode" {
+						return "", errors.New("balancer mode must not be set")
+					}
+					if args[0] == "balancer" && args[1] == "on" {
+						return "", nil
+					}
+				}
+				return "", errors.New("unknown command")
+			},
+		}
+		c.context.Executor = executor
+		err := c.enableBalancerModule()
+		assert.NoError(t, err)
+	})
+}
