@@ -7,6 +7,8 @@ set -ex
 CODEGEN_ERR="found codegen files! please run 'make codegen' and update your PR"
 MOD_ERR="changes found by mod.check. You may need to run make clean"
 CRD_ERR="changes found by 'make crds'. please run 'make crds' locally and update your PR"
+BUILD_ERR="changes found by make build', please commit your go.sum or other changed files"
+HELM_ERR="changes found by 'make gen-rbac'. please run 'make gen-rbac' locally and update your PR"
 
 #############
 # FUNCTIONS #
@@ -36,7 +38,13 @@ case "$1" in
   crd)
     validate "$CRD_ERR"
   ;;
+  build)
+    validate "$BUILD_ERR"
+  ;;
+  gen-rbac)
+    validate "$HELM_ERR"
+  ;;
   *)
-    echo $"Usage: $0 {codegen|modcheck|crd}"
+    echo $"Usage: $0 {codegen|modcheck|crd|build|gen-rbac}"
     exit 1
 esac
